/*===- TableGen'erated file -------------------------------------*- C++ -*-===*\
|*                                                                            *|
|* TypeDef Declarations                                                       *|
|*                                                                            *|
|* Automatically generated file, do not edit!                                 *|
|*                                                                            *|
\*===----------------------------------------------------------------------===*/

#ifdef GET_TYPEDEF_CLASSES
#undef GET_TYPEDEF_CLASSES


namespace mlir {
class DialectAsmParser;
class DialectAsmPrinter;
} // namespace mlir
namespace mlir {
namespace pdl {
  class AttributeType;
  class OperationType;
  class TypeType;
  class ValueType;

  class AttributeType : public ::mlir::Type::TypeBase<AttributeType, ::mlir::Type, ::mlir::TypeStorage> {
  public:
    /// Inherit some necessary constructors from 'TypeBase'.
    using Base::Base;

    static ::llvm::StringRef getMnemonic() { return "attribute"; }
  };

  class OperationType : public ::mlir::Type::TypeBase<OperationType, ::mlir::Type, ::mlir::TypeStorage> {
  public:
    /// Inherit some necessary constructors from 'TypeBase'.
    using Base::Base;

    static ::llvm::StringRef getMnemonic() { return "operation"; }
  };

  class TypeType : public ::mlir::Type::TypeBase<TypeType, ::mlir::Type, ::mlir::TypeStorage> {
  public:
    /// Inherit some necessary constructors from 'TypeBase'.
    using Base::Base;

    static ::llvm::StringRef getMnemonic() { return "type"; }
  };

  class ValueType : public ::mlir::Type::TypeBase<ValueType, ::mlir::Type, ::mlir::TypeStorage> {
  public:
    /// Inherit some necessary constructors from 'TypeBase'.
    using Base::Base;

    static ::llvm::StringRef getMnemonic() { return "value"; }
  };
} // namespace pdl
} // namespace mlir

#endif  // GET_TYPEDEF_CLASSES

