/* Autogenerated by mlir-tblgen; don't manually edit */
#ifdef GEN_PASS_CLASSES

//===----------------------------------------------------------------------===//
// TensorBufferize
//===----------------------------------------------------------------------===//

template <typename DerivedT>
class TensorBufferizeBase : public ::mlir::FunctionPass {
public:
  TensorBufferizeBase() : ::mlir::FunctionPass(::mlir::TypeID::get<DerivedT>()) {}
  TensorBufferizeBase(const TensorBufferizeBase &) : ::mlir::FunctionPass(::mlir::TypeID::get<DerivedT>()) {}

  /// Returns the command-line argument attached to this pass.
  ::llvm::StringRef getArgument() const override { return "tensor-bufferize"; }

  /// Returns the derived pass name.
  ::llvm::StringRef getName() const override { return "TensorBufferize"; }

  /// Support isa/dyn_cast functionality for the derived pass class.
  static bool classof(const ::mlir::Pass *pass) {
    return pass->getTypeID() == ::mlir::TypeID::get<DerivedT>();
  }

  /// A clone method to create a copy of this pass.
  std::unique_ptr<::mlir::Pass> clonePass() const override {
    return std::make_unique<DerivedT>(*static_cast<const DerivedT *>(this));
  }

  /// Return the dialect that must be loaded in the context before this pass.
  void getDependentDialects(::mlir::DialectRegistry &registry) const override {
    
  registry.insert<scf::SCFDialect>();

  }

protected:
};
#undef GEN_PASS_CLASSES
#endif // GEN_PASS_CLASSES
#ifdef GEN_PASS_REGISTRATION

//===----------------------------------------------------------------------===//
// TensorBufferize Registration
//===----------------------------------------------------------------------===//

inline void registerTensorBufferizePass() {
  ::mlir::registerPass("tensor-bufferize", "Bufferize the `tensor` dialect", []() -> std::unique_ptr<::mlir::Pass> {
    return mlir::createTensorBufferizePass();
  });
}

//===----------------------------------------------------------------------===//
// Tensor Registration
//===----------------------------------------------------------------------===//

inline void registerTensorPasses() {
  registerTensorBufferizePass();
}
#undef GEN_PASS_REGISTRATION
#endif // GEN_PASS_REGISTRATION
