/*===- TableGen'erated file -------------------------------------*- C++ -*-===*\
|*                                                                            *|
|* TypeDef Definitions                                                        *|
|*                                                                            *|
|* Automatically generated file, do not edit!                                 *|
|*                                                                            *|
\*===----------------------------------------------------------------------===*/

#ifdef GET_TYPEDEF_LIST
#undef GET_TYPEDEF_LIST

::mlir::pdl::AttributeType,
::mlir::pdl::OperationType,
::mlir::pdl::TypeType,
::mlir::pdl::ValueType

#endif  // GET_TYPEDEF_LIST

#ifdef GET_TYPEDEF_CLASSES
#undef GET_TYPEDEF_CLASSES

static ::mlir::Type generatedTypeParser(::mlir::MLIRContext *context, ::mlir::DialectAsmParser &parser, ::llvm::StringRef mnemonic) {
  if (mnemonic == ::mlir::pdl::AttributeType::getMnemonic()) return ::mlir::pdl::AttributeType::get(context);
  if (mnemonic == ::mlir::pdl::OperationType::getMnemonic()) return ::mlir::pdl::OperationType::get(context);
  if (mnemonic == ::mlir::pdl::TypeType::getMnemonic()) return ::mlir::pdl::TypeType::get(context);
  if (mnemonic == ::mlir::pdl::ValueType::getMnemonic()) return ::mlir::pdl::ValueType::get(context);
  return ::mlir::Type();
}

static ::mlir::LogicalResult generatedTypePrinter(::mlir::Type type, ::mlir::DialectAsmPrinter &printer) {
  return ::llvm::TypeSwitch<::mlir::Type, ::mlir::LogicalResult>(type)
    .Case<::mlir::pdl::AttributeType>([&](::mlir::pdl::AttributeType t) {
      printer << ::mlir::pdl::AttributeType::getMnemonic();
      return ::mlir::success();
    })
    .Case<::mlir::pdl::OperationType>([&](::mlir::pdl::OperationType t) {
      printer << ::mlir::pdl::OperationType::getMnemonic();
      return ::mlir::success();
    })
    .Case<::mlir::pdl::TypeType>([&](::mlir::pdl::TypeType t) {
      printer << ::mlir::pdl::TypeType::getMnemonic();
      return ::mlir::success();
    })
    .Case<::mlir::pdl::ValueType>([&](::mlir::pdl::ValueType t) {
      printer << ::mlir::pdl::ValueType::getMnemonic();
      return ::mlir::success();
    })
    .Default([](::mlir::Type) { return ::mlir::failure(); });
}

namespace mlir {
namespace pdl {
} // namespace pdl
} // namespace mlir
namespace mlir {
namespace pdl {
} // namespace pdl
} // namespace mlir
namespace mlir {
namespace pdl {
} // namespace pdl
} // namespace mlir
namespace mlir {
namespace pdl {
} // namespace pdl
} // namespace mlir

#endif  // GET_TYPEDEF_CLASSES

