/* Autogenerated by mlir-tblgen; don't manually edit */
#ifdef GEN_PASS_CLASSES

//===----------------------------------------------------------------------===//
// FuncBufferize
//===----------------------------------------------------------------------===//

template <typename DerivedT>
class FuncBufferizeBase : public ::mlir::OperationPass<ModuleOp> {
public:
  FuncBufferizeBase() : ::mlir::OperationPass<ModuleOp>(::mlir::TypeID::get<DerivedT>()) {}
  FuncBufferizeBase(const FuncBufferizeBase &) : ::mlir::OperationPass<ModuleOp>(::mlir::TypeID::get<DerivedT>()) {}

  /// Returns the command-line argument attached to this pass.
  ::llvm::StringRef getArgument() const override { return "func-bufferize"; }

  /// Returns the derived pass name.
  ::llvm::StringRef getName() const override { return "FuncBufferize"; }

  /// Support isa/dyn_cast functionality for the derived pass class.
  static bool classof(const ::mlir::Pass *pass) {
    return pass->getTypeID() == ::mlir::TypeID::get<DerivedT>();
  }

  /// A clone method to create a copy of this pass.
  std::unique_ptr<::mlir::Pass> clonePass() const override {
    return std::make_unique<DerivedT>(*static_cast<const DerivedT *>(this));
  }

  /// Return the dialect that must be loaded in the context before this pass.
  void getDependentDialects(::mlir::DialectRegistry &registry) const override {
    
  }

protected:
};

//===----------------------------------------------------------------------===//
// StdBufferize
//===----------------------------------------------------------------------===//

template <typename DerivedT>
class StdBufferizeBase : public ::mlir::FunctionPass {
public:
  StdBufferizeBase() : ::mlir::FunctionPass(::mlir::TypeID::get<DerivedT>()) {}
  StdBufferizeBase(const StdBufferizeBase &) : ::mlir::FunctionPass(::mlir::TypeID::get<DerivedT>()) {}

  /// Returns the command-line argument attached to this pass.
  ::llvm::StringRef getArgument() const override { return "std-bufferize"; }

  /// Returns the derived pass name.
  ::llvm::StringRef getName() const override { return "StdBufferize"; }

  /// Support isa/dyn_cast functionality for the derived pass class.
  static bool classof(const ::mlir::Pass *pass) {
    return pass->getTypeID() == ::mlir::TypeID::get<DerivedT>();
  }

  /// A clone method to create a copy of this pass.
  std::unique_ptr<::mlir::Pass> clonePass() const override {
    return std::make_unique<DerivedT>(*static_cast<const DerivedT *>(this));
  }

  /// Return the dialect that must be loaded in the context before this pass.
  void getDependentDialects(::mlir::DialectRegistry &registry) const override {
    
  registry.insert<scf::SCFDialect>();

  }

protected:
};

//===----------------------------------------------------------------------===//
// StdExpandOps
//===----------------------------------------------------------------------===//

template <typename DerivedT>
class StdExpandOpsBase : public ::mlir::FunctionPass {
public:
  StdExpandOpsBase() : ::mlir::FunctionPass(::mlir::TypeID::get<DerivedT>()) {}
  StdExpandOpsBase(const StdExpandOpsBase &) : ::mlir::FunctionPass(::mlir::TypeID::get<DerivedT>()) {}

  /// Returns the command-line argument attached to this pass.
  ::llvm::StringRef getArgument() const override { return "std-expand"; }

  /// Returns the derived pass name.
  ::llvm::StringRef getName() const override { return "StdExpandOps"; }

  /// Support isa/dyn_cast functionality for the derived pass class.
  static bool classof(const ::mlir::Pass *pass) {
    return pass->getTypeID() == ::mlir::TypeID::get<DerivedT>();
  }

  /// A clone method to create a copy of this pass.
  std::unique_ptr<::mlir::Pass> clonePass() const override {
    return std::make_unique<DerivedT>(*static_cast<const DerivedT *>(this));
  }

  /// Return the dialect that must be loaded in the context before this pass.
  void getDependentDialects(::mlir::DialectRegistry &registry) const override {
    
  }

protected:
};

//===----------------------------------------------------------------------===//
// TensorConstantBufferize
//===----------------------------------------------------------------------===//

template <typename DerivedT>
class TensorConstantBufferizeBase : public ::mlir::OperationPass<ModuleOp> {
public:
  TensorConstantBufferizeBase() : ::mlir::OperationPass<ModuleOp>(::mlir::TypeID::get<DerivedT>()) {}
  TensorConstantBufferizeBase(const TensorConstantBufferizeBase &) : ::mlir::OperationPass<ModuleOp>(::mlir::TypeID::get<DerivedT>()) {}

  /// Returns the command-line argument attached to this pass.
  ::llvm::StringRef getArgument() const override { return "tensor-constant-bufferize"; }

  /// Returns the derived pass name.
  ::llvm::StringRef getName() const override { return "TensorConstantBufferize"; }

  /// Support isa/dyn_cast functionality for the derived pass class.
  static bool classof(const ::mlir::Pass *pass) {
    return pass->getTypeID() == ::mlir::TypeID::get<DerivedT>();
  }

  /// A clone method to create a copy of this pass.
  std::unique_ptr<::mlir::Pass> clonePass() const override {
    return std::make_unique<DerivedT>(*static_cast<const DerivedT *>(this));
  }

  /// Return the dialect that must be loaded in the context before this pass.
  void getDependentDialects(::mlir::DialectRegistry &registry) const override {
    
  }

protected:
};
#undef GEN_PASS_CLASSES
#endif // GEN_PASS_CLASSES
#ifdef GEN_PASS_REGISTRATION

//===----------------------------------------------------------------------===//
// FuncBufferize Registration
//===----------------------------------------------------------------------===//

inline void registerFuncBufferizePass() {
  ::mlir::registerPass("func-bufferize", "Bufferize func/call/return ops", []() -> std::unique_ptr<::mlir::Pass> {
    return mlir::createFuncBufferizePass();
  });
}

//===----------------------------------------------------------------------===//
// StdBufferize Registration
//===----------------------------------------------------------------------===//

inline void registerStdBufferizePass() {
  ::mlir::registerPass("std-bufferize", "Bufferize the std dialect", []() -> std::unique_ptr<::mlir::Pass> {
    return mlir::createStdBufferizePass();
  });
}

//===----------------------------------------------------------------------===//
// StdExpandOps Registration
//===----------------------------------------------------------------------===//

inline void registerStdExpandOpsPass() {
  ::mlir::registerPass("std-expand", "Legalize std operations to be convertible to LLVM.", []() -> std::unique_ptr<::mlir::Pass> {
    return mlir::createStdExpandOpsPass();
  });
}

//===----------------------------------------------------------------------===//
// TensorConstantBufferize Registration
//===----------------------------------------------------------------------===//

inline void registerTensorConstantBufferizePass() {
  ::mlir::registerPass("tensor-constant-bufferize", "Bufferize tensor constants.", []() -> std::unique_ptr<::mlir::Pass> {
    return mlir::createTensorConstantBufferizePass();
  });
}

//===----------------------------------------------------------------------===//
// Standard Registration
//===----------------------------------------------------------------------===//

inline void registerStandardPasses() {
  registerFuncBufferizePass();
  registerStdBufferizePass();
  registerStdExpandOpsPass();
  registerTensorConstantBufferizePass();
}
#undef GEN_PASS_REGISTRATION
#endif // GEN_PASS_REGISTRATION
